package com.deltax.cpacker;

import java.io.*;
import java.util.*;
import java.util.jar.*;
import javax.swing.*;

/**
 * Excludes the classes already contained in given JAR file.
 * If another filter is provided, it is applied *before* checking
 * if a class is in the JAR.
 */
public class JarFileNameFilter implements ClassPacker.ClassNameFilter {
	
	HashSet names = new HashSet();
	ClassPacker.ClassNameFilter filter;
	
	public JarFileNameFilter(ClassPacker.ClassNameFilter filter, String jarFile) throws IOException {
		this.filter=filter;
		
		// Open the JAR file and read the class names
		JarFile jf = new JarFile(jarFile);
		
		ProgressMonitor jpm=new ProgressMonitor(null, "Extracting JAR file information..", "No class", 0, jf.size());
		
		Enumeration e = jf.entries();
		int i=0;
		while(e.hasMoreElements()) {
			JarEntry je = (JarEntry)e.nextElement();
			jpm.setNote(je.getName());
			jpm.setProgress(++i);
			
			if (jpm.isCanceled()) System.exit(-1);
						
			if (je.isDirectory()) continue;
			
			String className = je.getName();
			className= className.substring(0, className.lastIndexOf(".class")).replace('/','.');
			names.add(className);
		}
		
		jpm.close();
	}
	
	public JarFileNameFilter(String jarFile) throws IOException {
		this(null, jarFile);
	}
	
	/**
    * @return true if the className is accepted, false otherwise
    */
   public boolean accept(String className) {
   	// Check with the existing filter, if any
   	if (filter!=null) if (filter.accept(className) == false) return false;
   	// Check if the class is already in the JAR
   	return ! names.contains(className);
   }
}