//Source file: d:/data/classes/com/deltax/cpacker/gui/ClassPacker.java

package com.deltax.cpacker.gui;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import com.deltax.util.*;

/**
 * A GUI Shell which allows class packing just by drag-and-drop operation
 *
 * Requires ClassPacker 1.4; will work with previous versions, but some options might be missing.
 *
 * @version 1.0
 * @date Dec 1999, March 2000
 * @author dr. Cris Sadun
 */
public class ClassPacker extends JFrame 
{
	public static final String VERSION = "1.0";
	
   private JPanelOutputStream msgOut;
   private PrintStream out;
   private JList resultList;
   private JTextField cPathTF;
   private JCheckBox appendJCB;
   private JCheckBox askFNameJCB;
	private JCheckBox createManifestJCB;
   private JCheckBox createCFNreportJCB;
   private JCheckBox incJavaJCB;
   private JCheckBox incJavaExtJCB;
   private JCheckBox useZipJCB;
   
   // ClassPacker's invoked in a separate thread
   private boolean invokeInSeparateThread = false;
   
   // The executable name ("java" or "jre")
   private String jvmExecName = "java";
   // The path to the JVM (depending on JAVA_HOME settings);
   private String jvm;
   
   
   /**
    * @return 
    * @exception 
    * @author 
    * @see 
    * @roseuid 38E87C0B02AB
    */
   public ClassPacker() 
   { 
     super("ClassPacker GUI Shell v"+VERSION);
     
     addWindowListener(new WindowAdapter() {
     	public void windowClosing(WindowEvent e) {
     		exit();
     	}
     });
     
     setSize(500,500);
     setLocation(10,10);
     resultList=new JList(new String[] { "Drag the classes (.class files) here to pack them" } );
     resultList.setBackground(new Color(255,255,204));
     
     String jHome;
     
     if (System.getProperty("java.home")==null) {
     	jHome="not set";
		jvm=jvmExecName;
     } else {
     	jHome=System.getProperty("java.home");
     	//if (jHome.endsWith("jre")) jvmExecName="jre";
     	jvm=jHome+File.separator+"bin"+File.separator+jvmExecName;
     }
     
     
     
     // msgOut
     msgOut=new JPanelOutputStream(new String[] {
     	"ClassPacker GUI Shell v"+VERSION,
     	"(C) Copyright 1998, 99, 2000 Cristiano Sadun",
     	"All rights reserved",
     	"********************************************",
     	"Java home is "+jHome,
     	"Java virtual machine is "+jvm,
     	"",
     	"ClassPacker GUI Ready",
     	""
     });
     out=new PrintStream(msgOut);
     
     CPDTListener dtl = new CPDTListener();
     
     //resultList.
     resultList.setDropTarget(new DropTarget(this, DnDConstants.ACTION_COPY, dtl, true));
     
     // optionsPane
     JPanel optionsPane = new JPanel();
     optionsPane.setLayout(new BoxLayout(optionsPane, BoxLayout.Y_AXIS));
     optionsPane.add(appendJCB=new JCheckBox("Append to existing file",false));
     optionsPane.add(askFNameJCB=new JCheckBox("Ask file name on drag", false));
     optionsPane.add(createManifestJCB=new JCheckBox("Create Manifest file", true));
     optionsPane.add(createCFNreportJCB=new JCheckBox("Generate Class.forName() report", false));
     optionsPane.add(incJavaJCB=new JCheckBox("Include Java library classes", false));
     optionsPane.add(incJavaExtJCB=new JCheckBox("Include Java extensions classes", false));
     optionsPane.add(useZipJCB=new JCheckBox("Use .zip extension", false));
     
     
     // Classpath tf
     JPanel cPathP=new JPanel();
     cPathP.setLayout(new BoxLayout(cPathP,BoxLayout.X_AXIS));
     cPathP.add(new JLabel("ClassPath:"));
     cPathP.add(cPathTF=new JTextField(System.getProperty("java.class.path")));
     cPathTF.addActionListener(new ActionListener() {
     		public void actionPerformed(ActionEvent e) {
     			
     			try {
     				ClassPathValidator.checkClassPath(e.getActionCommand());
     				System.setProperty("java.class.path", e.getActionCommand());
     				out.println("Class path set to "+System.getProperty("java.class.path"));
     			} catch (PathSetValidator.InvalidPathSetException exc) {
     				JOptionPane.showMessageDialog(
						ClassPacker.this, 
						exc.getMessage(),
						"Invalid CLASSPATH", 
						JOptionPane.ERROR_MESSAGE);
					cPathTF.selectAll();
					cPathTF.requestFocus();
     			}
     		}
     	});
     
     getContentPane().setLayout(new java.awt.BorderLayout(2,2));
     getContentPane().add("North", cPathP);
     getContentPane().add("Center", resultList);
     getContentPane().add("East", optionsPane);
     getContentPane().add("South", msgOut.getPanel());
     JMenuBar mb = new JMenuBar();
     
     JMenu
     m = new JMenu("ClassPacker");
     JMenuItem 
     /*mi = new JMenuItem("Clear files list");
     mi.addActionListener(new ActionListener() {
     	public void actionPerformed(ActionEvent e) { 
     		
     	}
     });
     m.add(mi);
     */
     mi = new JMenuItem("Clear log panel");
     mi.addActionListener(new ActionListener() {
     	public void actionPerformed(ActionEvent e) { 
     		msgOut.clear();
     	}
     });
     m.add(mi);
     
     mi = new JMenuItem("Show ClassPacker engine data");
     m.add(mi);
     mi.addActionListener(new ActionListener() {
     	public void actionPerformed(ActionEvent e) { 
     		invokeCPMain("");
     	}
     });
     m.add(new JSeparator());
     mi = new JMenuItem("Exit");
     mi.addActionListener(new ActionListener() {
     	public void actionPerformed(ActionEvent e) { exit(); }
     });
     m.add(mi);
     mb.add(m);
     getRootPane().setJMenuBar(mb);
   }
   
   /**
    * @param args
    * @return void
    * @exception 
    * @author 
    * @see 
    * @roseuid 38E87C9A00C6
    */
   public static void main(String[] args) throws Exception
   { 
   	UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
   	com.deltax.cpacker.gui.ClassPacker cp = new com.deltax.cpacker.gui.ClassPacker();
      cp.setVisible(true);
   }
   
   private void exit() {
   	System.exit(0);
   }
   
   private void invokeCPMain(final String args) {
   	Vector v=new Vector();
   	StringTokenizer st=new StringTokenizer(args," ");
   	while(st.hasMoreTokens()) {
   		v.addElement(st.nextToken());
   	}
   	String _args[] = new String[v.size()];
   	v.copyInto(_args);
   	
   	Runnable rnbl = new Runnable() {
   		public void run() {
		   	try {
		   		String exec=jvm+" -classpath "+System.getProperty("java.class.path")+
			   					" com.deltax.cpacker.ClassPacker "+args;
		   		synchronized(out) { out.println(">Executing: "+exec); }
			   	Process p = Runtime.getRuntime().exec(exec);
			   	
			   	p.waitFor();
			   	InputStream is = p.getInputStream();
			   	synchronized(out) {
			   		out.println(">Finished: cpacker "+args); 
				   	while(is.available() >0) {
				   	 int c=is.read();
				   	 msgOut.write(c);
				   	}
			   	}
					//p.destroy();
			   } catch(IOException e) {
			   	e.printStackTrace();
			   	JOptionPane.showMessageDialog(
								ClassPacker.this, 
								e.getMessage(),
								"Exception occurred:"+e.getClass().getName(),
								JOptionPane.ERROR_MESSAGE);
			   } catch(InterruptedException e) {
			   	e.printStackTrace();
			   } 
			   
			   
			}
		 };
		 
		 if (invokeInSeparateThread) new Thread(rnbl).start();
		 else rnbl.run();
   }
   
	class CPDTListener implements DropTargetListener {
	    public void dragEnter(DropTargetDragEvent e) { 
	    	//System.out.println(e); 
	    	if (isAcceptable(e)) e.acceptDrag(DnDConstants.ACTION_COPY); 
	    	else e.rejectDrag();
	    }
	    public void dragOver(DropTargetDragEvent e) { 
	    	//System.out.println(e); 
	    	if (isAcceptable(e)) e.acceptDrag(DnDConstants.ACTION_COPY); 
	    	else e.rejectDrag(); 
	    }
		 public void dropActionChanged(DropTargetDragEvent e) { 
		 	//System.out.println(e); 
		 	if (isAcceptable(e)) e.acceptDrag(DnDConstants.ACTION_COPY); 
	    	else e.rejectDrag();
	    }
		 public void dragExit(DropTargetEvent e) { }
		 public void drop(DropTargetDropEvent e) { 
		 	DataFlavor[] flavors = e.getCurrentDataFlavors();
		 	DataFlavor df=null;
		 	for(int i=0;i < flavors.length; i++) {
				if (DataFlavor.javaFileListFlavor.equals(flavors[i]))
					df=flavors[i];
					break;
			}
			
			if (df==null) e.rejectDrop();
			else e.acceptDrop(DnDConstants.ACTION_COPY);
			
			try {
				java.util.List l = (java.util.List)e.getTransferable().getTransferData(df);
				File f = (File)l.get(0);
				if (! f.getName().toLowerCase().endsWith(".class")) {
					JOptionPane.showMessageDialog(
						ClassPacker.this, 
						"You may drop only (valid) .class files for packing",
						"Invalid Drop", 
						JOptionPane.ERROR_MESSAGE);
						e.dropComplete(false);
					return;
				} 
				
				try {
					// Let's open the class, and see what's inside
					// (CPoolReader is created here since we have to use the current class path every time)
					// Anyway, we have to explicitly override getClassName since the original 
					// CPoolReader.classfile was not tought for the job
					out.println("Reading class file..");
					
					CPoolReader cpr=new CPoolReader(System.getProperty("java.class.path"));
					classfile2 cf = new classfile2(cpr, f);
					
					//cpr.readClass(f.getName());
					String clsName = cf.getClassName();
					out.println("Done - class is "+clsName);
					
					out.println("Resolving class using "+System.getProperty("java.class.path")+"..");
					Class.forName(clsName);
					out.println("Done");
					
					// Okay, now let's call the underlying tool
					StringBuffer sb=new StringBuffer();
					
					if (! createManifestJCB.isSelected()) sb.append("-nmf ");
					if (appendJCB.isSelected()) sb.append("-append ");
					if (askFNameJCB.isSelected()) sb.append("-ask ");       
					if (createCFNreportJCB.isSelected()) sb.append("-fnr ");
					if (incJavaJCB.isSelected()) sb.append("-ij ");        
					if (incJavaExtJCB.isSelected()) sb.append("-ijx ");     
					if (useZipJCB.isSelected()) sb.append("-zip ");         

					sb.append(clsName);
					out.println("invoking cpacker "+sb.toString());
					
					invokeCPMain(sb.toString());
					
					e.dropComplete(true);
				} catch (ClassFormatError err) {
					err.printStackTrace(out);
					JOptionPane.showMessageDialog(
						ClassPacker.this, 
						f.getName()+" is not a valid java class file",
						"Class format error",
						JOptionPane.ERROR_MESSAGE);
						e.dropComplete(false);
				} catch (LinkageError err2) {
					err2.printStackTrace(out);
					JOptionPane.showMessageDialog(
						ClassPacker.this, 
						"Make sure your classpath is set up properly (linkage error).",
						"Class can't be resolved", 
						JOptionPane.ERROR_MESSAGE);
						e.dropComplete(false);
				} 
				
				
			} catch(Exception exc2) {
				exc2.printStackTrace(out);
				e.dropComplete(false);
			}
	    }
		 
		 private boolean isAcceptable(DropTargetDragEvent e) {
		 	DataFlavor[] flavors = e.getCurrentDataFlavors();
		 	DataFlavor df=null;
		 	for(int i=0;i < flavors.length; i++) {
				if (DataFlavor.javaFileListFlavor.equals(flavors[i]))
					df=flavors[i];
					break;
			}
			if (df==null) return false;
			return true;
		 }
		   
		};
}
